### Todo

- [ ] Accréditer les ressources photos
    - CC : Markdown icon created by Heztasia from Noun Project
    - CC : HTML icon created by Bagus Kusnandar

### Introduction

Public : connaissant les bases du dev web (HTML + serveur web), et les bases de *nix (GNU/Linux, Unix).

Mon site a plusieurs problèmes :

- Adhérence techno : traces de PHP pour factoriser des composants ;
- Complexité écriture : je dois écrire le contenu en PHP ;
- Déploiement maison : déploiement à base de scripts shells maison non standards.

J'ai décidé de refondre mon site afin de résoudre ces points de friction.

### Recherche

Mon principal objectif est de faciliter la production de contenus sur mon site, tout en utilisant un ou des logiciels libres. J'ai donc fait un peu de veille.

#### CMS

J'ai hésité à utiliser un CMS, comme wordpress, qui gère très bien la production de contenu, mais la migration aurait été difficile et je n'affectionne pas particulièrement les IHM complexes qui demandent beaucoup de clics. L'idéal serait de pouvoir écrire mes articles en Markdown.

#### Générateurs de site statiques

https://jekyllrb.com/

C'est la bonne solution, et je vous conseille d'employer Jekyll si vous souhaiter faire quelque chose de sérieux.

#### Revenir aux bases

Je connais pandoc par ailleurs.

J'ai utilisé GNU make pour compiler mes projets en C++.

### Refonte

#### Pandoc

##### Markdown en HTML

Besoin de base.

##### HTML indépendant

Version standalone.

Permet aussi les pages HTML customisées légères.

Pour le CV, je peux écrire du HTML simple et profiter  du templating pour le rendre indépendant.

##### Templating

Pas besoin de Jinja !

Créer une structure commune `<head>` etc. et importer les styles.

##### Les métadonnées

Permet d'enrichir le template. Avant j'utilisais une variable PHP :

```php
<?php
    $title = "Contact";
    include("include/head.php");
?>
```

Basique : titre, auteur

Généré : version, date

Avancé : itération sur une liste de composants (articles, projets)

#### GNU Make

##### Convertir un fichier source

Les fichiers sources sont dans le dossier `src` et possèdent la particule `src` dans leur nom. J'ai écrit une recette pour appliquer pandoc sur ces fichiers :

```make
PANDOC_GEN := pandoc --standalone --wrap=none --template src/templates/main.html --metadata-file src/meta.yml

%.gen.html: %.src.md
    $(PANDOC_GEN) --output $@ $<

%.gen.html: %.src.html
    $(PANDOC_GEN) --output $@ $<
```

Ce qui donne :

```shell
➤ make src/cv/content.gen.html -n
```

```out
pandoc --standalone --wrap=none --template src/templates/main.html --metadata-file src/meta.yml --output src/cv/content.gen.html src/cv/content.src.html
```

On obtient donc les sorties générées des fichiers sources.

- `content.src.md` -> `content.gen.html`
- `content.src.html` -> `content.gen.html`

##### Produire tous les fichiers sources

Et nettoyage.

##### Produire un dossier distribuable

Et nettoyage.

##### Automatiser le déploiement distant

### Limites

#### Template HTML

Pandoc n'est pas un réel moteur de template. Si bien que passer du HTML à du HTML n'est pas fidèle. Pandoc utilise une représentation du document en interne qui fait que le HTML diverge sur certains points.

Il n'est d'ailleurs pas possible d'ajouter des composants maison dans notre présentation.

#### Nombre de dépendances

+ GNU/Make
    + `pandoc`
    + `yq`
    + `git`
    + shell
        + date
- PHP

### Pour aller plus loin

#### Minimiser l'empreinte du site

Filtrer les fichiers inclus dans un fichier source et les indiquer comme étant des dépendances.

Détecter et supprimer les fichiers inutiles.

#### Minifier les fichiers déployés

Enlever les caractères blancs, les commentaires, supprimer le CSS ou JS inutilisé.

#### Utiliser un watcher et auto-compiler à chaque modification

Déclencher un `make install` à chaque sauvegarde de fichier.

### Conclusion

J'ai donc construit un système plus léger au niveau de la rédaction de contenu qui me permettra de facilement produire du nouveau contenu en markdown. J'ai troqué la dépendance serveur PHP contre deux dépendances au build, pandoc et GNU Make. L'opération est donc un succès et j'utilise aujourd'hui ce projet en production.

J'ai gagné en confort, en employant des technologies stables que je connaissais par ailleurs.

Mais cela reste un projet d'exploration que je ne recommande pas forcément pour un usage pro. Ces choix souffrent de quelques limites : j'arrive aux limites du moteur de template de pandoc. Et contrairement à Jekyll, je ne dispose pas (encore) de rechargement à chaud. Pour les besoins de mon site ce n'est pas très grave car il est déjà construit et je compte seulement rajouter des projets ou articles, mais si j'ai besoin de faire quelque chose de sérieux j'utiliserai Jekyll au autre technologie destinée à cet usage.

Merci !

#### Refs

- Outils
    - Pandoc
    - GNU Make
    - CMS
        - [Wagtail](https://wagtail.org/features/)
        - [Wordpress](https://wordpress.org/download/)
    - Static generators
        - [Jekyll](https://jekyllrb.com/)
        - [HUGO](https://gohugo.io/)
        - [Grav](https://getgrav.org/features)
- Moi
    - Mon site
    - Mon code source

